@@ -3,9 +3,6 @@
 #include "bl_dlfcn.h"
 
 #include <stdio.h>  /* NULL */
-#include <string.h> /* strlen */
-
-#include "bl_mem.h" /* alloca() */
 
 /* --- global functions --- */
 
@@ -15,7 +12,7 @@
 
 bl_dl_handle_t bl_dl_open(const char *dirpath, const char *name) { return NULL; }
 
-int bl_dl_close(bl_dl_handle_t handle) { return 0; }
+void bl_dl_close(bl_dl_handle_t handle) {}
 
 void *bl_dl_func_symbol(bl_dl_handle_t handle, const char *symbol) { return NULL; }
 
