@@ -184,7 +184,7 @@ static size_t convert_to_johab(ef_conv_t *conv, u_char *dst, size_t dst_size,
 
 static void conv_init(ef_conv_t *conv) {}
 
-static void conv_delete(ef_conv_t *conv) { free(conv); }
+static void conv_destroy(ef_conv_t *conv) { free(conv); }
 
 /* --- global functions --- */
 
@@ -197,7 +197,7 @@ ef_conv_t *ef_johab_conv_new(void) {
 
   conv->convert = convert_to_johab;
   conv->init = conv_init;
-  conv->delete = conv_delete;
+  conv->destroy = conv_destroy;
   conv->illegal_char = NULL;
 
   return conv;
