@@ -1,6 +1,6 @@
 #!/usr/pkg/bin/perl
 
-# source file is vscii.txt
+# source file is TCVN5712-1.txt
 
 use ucs_mappings () ;
 
@@ -11,7 +11,7 @@ my @tcvn ;
 
 foreach my $line (@all_lines)
 {
-	if( $line =~ /^(0x[0-9a-z]*).*$/)
+	if( $line =~ /^(0x[0-9A-Z]*).*$/)
 	{
 		my $code = oct $1 ;
 
@@ -26,7 +26,7 @@ foreach my $line (@all_lines)
 	}
 }
 
-ucs_mappings::parse( '^(0x[0-9a-z]+)[	 ]+(0x[0-9a-z]+).*$' , \@tcvn) ;
+ucs_mappings::parse( '^(0x[0-9A-Z]+)[	 ]+(0x[0-9A-Z]+).*$' , \@tcvn) ;
 
-ucs_mappings::output_table_to_ucs( ${cs} , 2) ;
-ucs_mappings::output_table_ucs_to( ${cs} , 1) ;
+ucs_mappings::output_table_to_ucs( ${cs}) ;
+ucs_mappings::output_table_ucs_to( ${cs}) ;
