@@ -5,9 +5,11 @@
 
 #include <gtk/gtk.h>
 
-#define MC_SPACES 2
+#define MC_SPACES 4
 #define MC_SPACE_LINE 0
 #define MC_SPACE_LETTER 1
+#define MC_SPACE_BASELINE 2
+#define MC_SPACE_UNDERLINE 3
 
 GtkWidget *mc_space_config_widget_new(int id);
 
