@@ -953,7 +953,7 @@ ui_display_t **ui_get_opened_displays(u_int *num) {
 int ui_display_fd(ui_display_t *disp) { return fileno(disp->display->fp); }
 
 int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, int hint,
-                         char *app_name, Window parent_window /* Ignored */
+                         char *app_name, char *wm_role, Window parent_window /* Ignored */
                          ) {
   void *p;
 
