@@ -279,7 +279,13 @@ static int copy_area(ui_window_t *win, Pixmap src, int src_x, /* can be minus */
   }
 #endif
 
-  if (dst_x >= (int)win->width + hmargin || dst_y >= (int)win->height + vmargin) {
+  if (width == 0 || height == 0) {
+    /* This can happen in setting a wall paper. */
+    return 0;
+  }
+
+  if (dst_x >= (int)win->width + hmargin - right_margin ||
+      dst_y >= (int)win->height + vmargin - bottom_margin) {
     return 0;
   }
 
@@ -437,6 +443,7 @@ int ui_window_init(ui_window_t *win, u_int width, u_int height, u_int min_width,
   win->min_height = min_height;
   win->width_inc = width_inc;
   win->height_inc = height_inc;
+  win->sizehint_flag = SIZEHINT_WIDTH|SIZEHINT_HEIGHT;
   win->hmargin = 0 /* hmargin */;
   win->vmargin = 0 /* vmargin */;
 
@@ -1415,7 +1422,7 @@ void ui_set_window_name(ui_window_t *win, u_char *name) {
     }
     fwrite("\x07", 1, 1, win->disp->display->fp);
 
-    (*parser->delete)(parser);
+    (*parser->destroy)(parser);
   }
 }
 
@@ -1446,7 +1453,7 @@ void ui_set_icon_name(ui_window_t *win, u_char *name) {
     }
     fwrite("\x07", 1, 1, win->disp->display->fp);
 
-    (*parser->delete)(parser);
+    (*parser->destroy)(parser);
   }
 }
 
