@@ -2,7 +2,20 @@
 
 #include "ui_virtual_kbd.h"
 
+#ifdef NO_IMAGE
+
+int ui_virtual_kbd_hide(void) { return 0; }
+
+int ui_is_virtual_kbd_event(ui_display_t *disp, XButtonEvent *bev) { return 0; }
+
+int ui_virtual_kbd_read(XKeyEvent *kev, XButtonEvent *bev) { return 0; }
+
+ui_window_t *ui_is_virtual_kbd_area(int y) { return NULL; }
+
+#else /* NO_IMAGE */
+
 #include <pobl/bl_debug.h>
+#include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 
 #include "../ui_imagelib.h"
 
@@ -303,12 +316,12 @@ static int start_virtual_kbd(ui_display_t *disp) {
 
 error:
   if (normal_pixmap) {
-    ui_delete_image(disp->display, normal_pixmap);
+    ui_destroy_image(disp->display, normal_pixmap);
     normal_pixmap = NULL;
   }
 
   if (pressed_pixmap) {
-    ui_delete_image(disp->display, pressed_pixmap);
+    ui_destroy_image(disp->display, pressed_pixmap);
     pressed_pixmap = NULL;
   }
 
@@ -323,10 +336,10 @@ int ui_virtual_kbd_hide(void) {
   }
 
 #if 0
-  ui_delete_image(kbd_win->disp->display, normal_pixmap);
+  ui_destroy_image(kbd_win->disp->display, normal_pixmap);
   normal_pixmap = NULL;
 
-  ui_delete_image(kbd_win->disp->display, pressed_pixmap);
+  ui_destroy_image(kbd_win->disp->display, pressed_pixmap);
   pressed_pixmap = NULL;
 #endif
 
@@ -422,7 +435,7 @@ int ui_virtual_kbd_read(XKeyEvent *kev, XButtonEvent *bev) {
     x = bev->x - x_off;
 
     for (count = 0, key_group = kbd_key_groups;
-         count < sizeof(kbd_key_groups) / sizeof(kbd_key_groups[0]); count++, key_group++) {
+         count < BL_ARRAY_SIZE(kbd_key_groups); count++, key_group++) {
       if (y < key_group->top) {
         break;
       }
@@ -490,3 +503,5 @@ ui_window_t *ui_is_virtual_kbd_area(int y) {
     return NULL;
   }
 }
+
+#endif
