@@ -13,6 +13,7 @@
 #include <pobl/bl_debug.h>
 #include <pobl/bl_mem.h> /* alloca */
 #include <pobl/bl_str.h> /* bl_str_to_uint */
+#include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 #include <mef/ef_ucs4_map.h>
 #include <mef/ef_ucs_property.h>
 
@@ -106,7 +107,7 @@ static int parse_font_name(
         char *styles[] = { "italic",  "bold", "medium", "oblique", "light", "semi-bold", "heavy",
                            "semi-condensed", };
 
-        for (count = 0; count < sizeof(styles) / sizeof(styles[0]); count++) {
+        for (count = 0; count < BL_ARRAY_SIZE(styles); count++) {
           size_t len_v;
 
           len_v = strlen(styles[count]);
@@ -164,10 +165,6 @@ static int parse_font_name(
 
 /* --- global functions --- */
 
-void ui_compose_dec_special_font(void) {
-  /* Do nothing for now in quartz. */
-}
-
 /* Undocumented */
 bool CGFontGetGlyphsForUnichars(CGFontRef, u_int16_t[], CGGlyph[], size_t);
 
@@ -175,10 +172,16 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
                        ui_font_present_t font_present, const char *fontname, u_int fontsize,
                        u_int col_width, int use_medium_for_bold, int letter_space) {
   ui_font_t *font;
+  ef_charset_t cs;
   char *font_family;
   u_int percent;
   u_int cols;
 
+  cs = FONT_CS(id);
+  if (cs == DEC_SPECIAL && fontname == NULL) {
+    return NULL;
+  }
+
   if (type_engine != TYPE_XCORE ||
       (font = calloc(1, sizeof(ui_font_t) + sizeof(XFontStruct))) == NULL) {
 #ifdef DEBUG
@@ -199,7 +202,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
     cols = 1;
   }
 
-  if (IS_ISCII(FONT_CS(font->id)) || FONT_CS(font->id) == ISO10646_UCS4_1_V) {
+  if (IS_ISCII(cs) || cs == ISO10646_UCS4_1_V) {
     /*
      * For exampe, 'W' width and 'l' width of OR-TTSarala font for ISCII_ORIYA
      * are the same by chance, though it is actually a proportional font.
@@ -348,8 +351,6 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
     }
   }
 
-  font->decsp_font = NULL;
-
   if (size_attr) {
     font->width *= 2;
     font->x_off *= 2;
@@ -369,6 +370,11 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
   return font;
 }
 
+ui_font_t *ui_font_new_for_decsp(Display *display, vt_font_t id, u_int width, u_int height,
+                                 u_int ascent) {
+  return NULL;
+}
+
 void ui_font_destroy(ui_font_t *font) {
 #ifdef USE_OT_LAYOUT
   if (font->ot_font) {
