@@ -8,9 +8,11 @@ void TEST_bl_args(void);
 void TEST_bl_map(void);
 void TEST_bl_path(void);
 void TEST_bl_str(void);
+void TEST_encodefilter(void);
 void TEST_ui_font_config(void);
 void TEST_ui_emoji(void);
 void TEST_vt_parser(void);
+void TEST_vt_termcap(void);
 
 #ifndef NO_IMAGE
 void TEST_sixel_realloc_pixels(void);
@@ -27,9 +29,11 @@ void test(void) {
   TEST_bl_map();
   TEST_bl_path();
   TEST_bl_str();
+  TEST_encodefilter();
   TEST_ui_font_config();
   TEST_ui_emoji();
   TEST_vt_parser();
+  TEST_vt_termcap();
 
 #ifndef NO_IMAGE
   TEST_sixel_realloc_pixels();
