@@ -8,7 +8,6 @@
 #include <string.h>       /* memset/memcpy */
 #include <sys/time.h>     /* timeval */
 #include <unistd.h>       /* select */
-#include <errno.h>
 #include <pobl/bl_file.h> /* bl_file_set_cloexec */
 #endif
 
@@ -54,6 +53,7 @@ static void receive_next_event(void) {
   int ret;
   fd_set read_fds;
   struct timeval tval;
+  int is_sending_data = 0;
   ui_display_t **displays;
   u_int num_displays;
 #ifdef USE_LIBSSH2
@@ -64,6 +64,10 @@ static void receive_next_event(void) {
 #endif
   num_terms = vt_get_all_terms(&terms);
 
+#ifdef USE_BEOS
+  beos_lock();
+#endif
+
   while (1) {
 /* on Linux tv_usec,tv_sec members are zero cleared after select() */
 #ifdef KEY_REPEAT_BY_MYSELF
@@ -149,6 +153,12 @@ static void receive_next_event(void) {
         if (ptyfd > maxfd) {
           maxfd = ptyfd;
         }
+
+        if (vt_term_is_sending_data(terms[count])) {
+          tval.tv_usec = 0;
+          is_sending_data = 1;
+          vt_term_parse_vt100_sequence(terms[count]);
+        }
       }
     }
 
@@ -162,6 +172,10 @@ static void receive_next_event(void) {
       }
     }
 
+#ifdef USE_BEOS
+    beos_unlock();
+#endif
+
 #ifdef __HAIKU__
     if (vt_check_sig_child()) {
       return;
@@ -169,16 +183,26 @@ static void receive_next_event(void) {
 #endif
     if ((ret = select(maxfd + 1, &read_fds, NULL, NULL, &tval)) != 0) {
       if (ret < 0) {
-        if (errno != EINTR) {
 #ifdef DEBUG
-          bl_debug_printf(BL_DEBUG_TAG " error happened in select.\n");
+        bl_debug_printf(BL_DEBUG_TAG " error happened in select.\n");
 #endif
 
-          return;
-        }
-      } else {
-        break;
+        return;
       }
+
+      break;
+    }
+
+#ifdef USE_BEOS
+    /* UI thread might create a new vt_term by pressing Ctrl+F1 key and so on. */
+    num_terms = vt_get_all_terms(&terms);
+    beos_lock();
+#endif
+
+    if (is_sending_data) {
+      is_sending_data = 0;
+
+      continue;
     }
 
 #ifdef KEY_REPEAT_BY_MYSELF
@@ -189,10 +213,6 @@ static void receive_next_event(void) {
     display_idling_wait = 4;
 #endif
 
-#ifdef USE_BEOS
-    beos_lock();
-#endif
-
     for (count = 0; count < num_displays; count++) {
       ui_display_idling(displays[count]);
     }
@@ -209,16 +229,11 @@ static void receive_next_event(void) {
         (*additional_fds[count].handler)();
       }
     }
-
-#ifdef USE_BEOS
-    beos_unlock();
-
-    /* UI thread might create a new vt_term by pressing Ctrl+F1 key and so on. */
-    num_terms = vt_get_all_terms(&terms);
-#endif
   }
 
 #ifdef USE_BEOS
+  /* UI thread might create a new vt_term by pressing Ctrl+F1 key and so on. */
+  num_terms = vt_get_all_terms(&terms);
   beos_lock();
 #endif
 
@@ -324,6 +339,9 @@ int ui_event_source_process(void) {
 
   for (count = 0; count < num_terms; count++) {
     vt_term_parse_vt100_sequence(terms[count]);
+    if (vt_term_is_sending_data(terms[count])) {
+      ui_display_trigger_pty_read();
+    }
   }
 
 #ifndef USE_WIN32API
