@@ -13,6 +13,7 @@
 #include <vt_term_manager.h>
 #include "ui_selection_encoding.h"
 #include "ui_emoji.h"
+#include "ui_sb_view_factory.h"
 
 #if defined(__ANDROID__) || defined(USE_QUARTZ) || defined(USE_WIN32API)
 /* Shrink unused memory */
@@ -129,6 +130,7 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
   bl_conf_add_opt(conf, 'M', "im", 0, "input_method",
                   "input method (xim/kbd/uim/m17nlib/scim/ibus/fcitx/canna/wnn/skk/iiimf/none) [xim]");
   bl_conf_add_opt(conf, 'N', "name", 0, "app_name", "application name");
+  bl_conf_add_opt(conf, '\0', "role", 0, "wm_role", "application role [none]");
   bl_conf_add_opt(conf, 'O', "sbmod", 0, "scrollbar_mode",
                   "scrollbar mode (none/left/right/autohide) [none]");
   bl_conf_add_opt(conf, 'P', "clip", 1, "use_clipboard",
@@ -331,6 +333,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "directory to save received files [~/.mlterm/recv]");
   bl_conf_add_opt(conf, '\0', "fk", 1, "format_other_keys",
                   "send modified keys as parameter for CSI u [false]");
+  bl_conf_add_opt(conf, '\0', "sdpr", 0, "simple_scrollbar_dpr",
+                  "device pixel ratio for simple scrollbar [1]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -392,6 +396,9 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   if ((value = bl_conf_get_value(conf, "icon_name"))) {
     main_config->icon_name = strdup(value);
   }
+  if ((value = bl_conf_get_value(conf, "wm_role"))) {
+    main_config->wm_role = strdup(value);
+  }
 #endif
 
 /* BACKWARD COMPAT (3.1.7 or before) */
@@ -1428,6 +1435,14 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     vt_set_recv_dir(value);
   }
 
+  if ((value = bl_conf_get_value(conf, "simple_scrollbar_dpr"))) {
+    int dpr;
+
+    if (bl_str_to_int(&dpr, value) && dpr > 0) {
+      ui_sb_view_set_dpr(dpr);
+    }
+  }
+
 #ifdef USE_IM_CURSOR_COLOR
   if ((value = bl_conf_get_value(conf, "im_cursor_color"))) {
     if (*value) {
@@ -1471,6 +1486,7 @@ void ui_main_config_final(ui_main_config_t *main_config) {
   free(main_config->app_name);
   free(main_config->title);
   free(main_config->icon_name);
+  free(main_config->wm_role);
   free(main_config->term_type);
   free(main_config->scrollbar_view_name);
   free(main_config->pic_file_path);
