@@ -142,16 +142,14 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
   Window window;
   u_int width_pix;
   u_int height_pix;
-  char *env[7]; /* MLTERM,TERM,WINDOWID,WAYLAND_DISPLAY,DISPLAY,COLORFGBG,NULL */
+  char *env[8]; /* MLTERM,TERM,WINDOWID,WAYLAND_DISPLAY,DISPLAY,COLORFGBG,COLORTERM,NULL */
   char **env_p;
   char wid_env[9 + DIGIT_STR_LEN(Window) + 1]; /* "WINDOWID="(9) + [32bit digit] + NULL(1) */
   char *disp_env;
   char *term_env;
   char *uri;
   char *pass;
-#ifdef USE_LIBSSH2
   char *privkey;
-#endif
   int ret;
 
   display = win->disp->name;
@@ -195,6 +193,7 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
     *(env_p++) = term_env;
   }
 
+  *(env_p++) = "COLORTERM=truecolor";
   *(env_p++) = "COLORFGBG=default;default";
 
   /* NULL terminator */
@@ -204,6 +203,8 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
   pass = NULL;
 #ifdef USE_LIBSSH2
   privkey = main_config.private_key;
+#else
+  privkey = NULL;
 #endif
 
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
@@ -402,9 +403,11 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 
     free(pass);
 
+#ifdef USE_LIBSSH2
     if (privkey != main_config.private_key) {
       free(privkey);
     }
+#endif
   }
 #endif
 
@@ -634,7 +637,7 @@ static ui_screen_t *open_screen_intern(char *disp_name, vt_term_t *term, ui_layo
     }
 
     if (!ui_display_show_root(disp, root, main_config.x, main_config.y, main_config.geom_hint,
-                              main_config.app_name, main_config.parent_window)) {
+                              main_config.app_name, main_config.wm_role, main_config.parent_window)) {
 #ifdef DEBUG
       bl_warn_printf(BL_DEBUG_TAG " ui_display_show_root() failed.\n");
 #endif
@@ -801,8 +804,10 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
     if (!main_config.show_dialog) {
       default_server = main_config.default_server;
       main_config.default_server = vt_term_get_uri(screen->term);
+#ifdef USE_LIBSSH2
       privkey = main_config.private_key;
       main_config.private_key = vt_term_get_privkey(screen->term);
+#endif
     }
 
     if ((new_cmd_line = vt_term_get_cmd_line(screen->term)) &&
@@ -833,7 +838,9 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
     if (!main_config.show_dialog) {
       main_config.default_server = default_server;
+#ifdef USE_LIBSSH2
       main_config.private_key = privkey;
+#endif
     }
 
     if (new_cmd_line) {
@@ -961,7 +968,9 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
   vt_char_encoding_t encoding;
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   char *default_server;
+#ifdef USE_LIBSSH2
   char *privkey;
+#endif
   char *new_cmd_line;
   char *new_cmd_line_dup;
   char *cmd_path;
@@ -974,8 +983,10 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
   if (!show_dialog) {
     default_server = main_config.default_server;
     main_config.default_server = vt_term_get_uri(cur_screen->term);
+#ifdef USE_LIBSSH2
     privkey = main_config.private_key;
     main_config.private_key = vt_term_get_privkey(cur_screen->term);
+#endif
   }
 
   if ((new_cmd_line = vt_term_get_cmd_line(cur_screen->term)) &&
@@ -1006,7 +1017,9 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   if (!show_dialog) {
     main_config.default_server = default_server;
+#ifdef USE_LIBSSH2
     main_config.private_key = privkey;
+#endif
   }
 
   if (new_cmd_line) {
