@@ -10,6 +10,7 @@
 #include <pobl/bl_file.h>
 #include <pobl/bl_conf_io.h>
 #include <pobl/bl_str.h> /* strdup */
+#include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 
 #ifndef CommandMask
 #define CommandMask (0)
@@ -58,6 +59,9 @@ static key_func_table_t key_func_table[] = {
   { "SCROLL_UP_TO_MARK", SCROLL_UP_TO_MARK, },
   { "SCROLL_DOWN_TO_MARK", SCROLL_DOWN_TO_MARK, },
   { "INSERT_SELECTION", INSERT_SELECTION, },
+#if defined(USE_XLIB) || defined(USE_WAYLAND)
+  { "INSERT_CLIPBOARD", INSERT_CLIPBOARD, },
+#endif
   { "RESET", RESET, },
   { "COPY_MODE", COPY_MODE, },
   { "SET_MARK", SET_MARK, },
@@ -172,6 +176,11 @@ void ui_shortcut_init(ui_shortcut_t *shortcut) {
     { XK_Insert, ShiftMask, 1, },
 #endif
 
+#if defined(USE_XLIB) || defined(USE_WAYLAND)
+    /* INSERT_CLIPBOARD */
+    { 0, 0, 0, },
+#endif
+
     /* RESET */
     { XK_Pause, 0, 1, },
 
@@ -361,7 +370,7 @@ int ui_shortcut_parse(ui_shortcut_t *shortcut, char *key, char *oper) {
     return 0;
   }
 
-  for (count = 0; count < sizeof(key_func_table) / sizeof(key_func_table_t); count++) {
+  for (count = 0; count < BL_ARRAY_SIZE(key_func_table); count++) {
     ui_key_t *map_entry;
 
     map_entry = shortcut->map + key_func_table[count].func;
@@ -401,7 +410,7 @@ int ui_shortcut_parse(ui_shortcut_t *shortcut, char *key, char *oper) {
     shortcut->str_map = str_map;
   } else {
   replace_shortcut_map:
-    for (count = 0; count < sizeof(key_func_table) / sizeof(key_func_table_t); count++) {
+    for (count = 0; count < BL_ARRAY_SIZE(key_func_table); count++) {
       if (strcmp(oper, key_func_table[count].name) == 0) {
         if (strcmp(key, "UNUSED") == 0) {
           shortcut->map[key_func_table[count].func].is_used = 0;
