@@ -20,6 +20,9 @@
 #ifdef ZXDG_SHELL_V6
 #include "xdg-shell-unstable-v6-client-protocol.h"
 #endif
+#ifndef COMPAT_LIBVTE
+#include "xdg-decoration-unstable-v1-client-protocol.h"
+#endif
 
 typedef int KeyCode; /* Same as type of wparam */
 typedef int KeySym;  /* Same as type of wparam */
@@ -137,6 +140,7 @@ typedef struct {
   int shell_type; /* 0: Not determined, 1: wl_shell, 2: zxdg_shell, 3: xdg_shell */
 #else
   struct wl_shell *shell;
+  struct zxdg_decoration_manager_v1 *decoration_manager;
 #ifdef XDG_SHELL
   struct xdg_wm_base *xdg_shell;
 #endif
@@ -203,6 +207,7 @@ typedef struct {
   struct xdg_toplevel *xdg_toplevel;
   struct xdg_popup *xdg_popup;
   int xdg_surface_configured;
+  struct zxdg_toplevel_decoration_v1 *toplevel_decoration;
 #endif
 #endif /* COMPAT_LIBVTE */
 
@@ -488,7 +493,7 @@ typedef struct {
 
 int XParseGeometry(char *str, int *x, int *y, unsigned int *width, unsigned int *height);
 
-KeySym XStringToKeysym(char *str);
+KeySym XStringToKeysym(const char *str);
 
 /* === Platform dependent options === */
 
