@@ -384,6 +384,12 @@ static void registry_global(void *data, struct wl_registry *registry, uint32_t n
     wlserv->xsel_device_manager =
       wl_registry_bind(registry, name, &gtk_primary_selection_device_manager_interface, 1);
   }
+#ifndef COMPAT_LIBVTE
+  else if (strcmp(interface, "zxdg_decoration_manager_v1") == 0) {
+    wlserv->decoration_manager =
+      wl_registry_bind(registry, name, &zxdg_decoration_manager_v1_interface, 1);
+  }
+#endif
 #ifdef __DEBUG
   else {
     bl_debug_printf("Unknown interface: %s\n", interface);
@@ -1322,7 +1328,9 @@ static void shell_surface_configure(void *data, struct wl_shell_surface *shell_s
                   disp->display->width, disp->display->height, width, height);
 #endif
 
-  if (check_resize(disp->display->width, disp->display->height, &width, &height,
+  /* mutter and gnome-shell 41.3 can invoke configure event with width == 0 and height == 0. */
+  if (width != 0 && height != 0 &&
+      check_resize(disp->display->width, disp->display->height, &width, &height,
                    total_min_width(disp->roots[0]), total_min_height(disp->roots[0]),
                    max_width_inc(disp->roots[0]), max_height_inc(disp->roots[0]),
                    disp->display->is_resizing)) {
@@ -2087,6 +2095,12 @@ static void close_wl_display(ui_wlserv_t *wlserv) {
     wlserv->xsel_source = NULL;
   }
 
+#ifndef COMPAT_LIBVTE
+  if (wlserv->decoration_manager) {
+    zxdg_decoration_manager_v1_destroy(wlserv->decoration_manager);
+  }
+#endif
+
 #ifdef COMPAT_LIBVTE
   wl_subcompositor_destroy(wlserv->subcompositor);
 #else
@@ -2198,6 +2212,12 @@ static void close_display(ui_display_t *disp) {
 #endif
 #ifdef XDG_SHELL
   if (disp->display->xdg_surface) {
+#ifndef COMPAT_LIBVTE
+    if (disp->display->toplevel_decoration) {
+      zxdg_toplevel_decoration_v1_destroy(disp->display->toplevel_decoration);
+    }
+#endif
+
     destroy_shm_buffer(disp->display);
     disp->display->buffer = NULL;
     if (disp->display->xdg_toplevel) {
@@ -2350,6 +2370,20 @@ static int flush_display(Display *display) {
   }
 }
 
+#ifndef COMPAT_LIBVTE
+static void decoration_configure(void *data,
+                                 struct zxdg_toplevel_decoration_v1 *zxdg_toplevel_decoration_v1,
+                                 uint32_t mode) {
+#ifdef __DEBUG
+  bl_debug_printf("decoration mode: %d\n", mode);
+#endif
+}
+
+static const struct zxdg_toplevel_decoration_v1_listener decoration_listener = {
+  decoration_configure
+};
+#endif
+
 static void create_surface(ui_display_t *disp, int x, int y, u_int width, u_int height,
                            char *app_name) {
   Display *display = disp->display;
@@ -2435,6 +2469,18 @@ static void create_surface(ui_display_t *disp, int x, int y, u_int width, u_int
 #if 0
       xdg_surface_set_window_geometry(display->xdg_surface, x, y, width, height);
 #endif
+
+#ifndef COMPAT_LIBVTE
+      if (wlserv->decoration_manager) {
+        display->toplevel_decoration =
+          zxdg_decoration_manager_v1_get_toplevel_decoration(wlserv->decoration_manager,
+                                                             display->xdg_toplevel);
+        zxdg_toplevel_decoration_v1_add_listener(display->toplevel_decoration,
+                                                 &decoration_listener, display);
+        zxdg_toplevel_decoration_v1_set_mode(display->toplevel_decoration,
+                                             ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
+      }
+#endif
     }
     wl_surface_commit(display->surface);
   } else
@@ -2578,7 +2624,7 @@ ui_display_t **ui_get_opened_displays(u_int *num) {
 int ui_display_fd(ui_display_t *disp) { return wl_display_get_fd(disp->display->wlserv->display); }
 
 int ui_display_show_root(ui_display_t *disp, ui_window_t *root, int x, int y, int hint,
-                         char *app_name, Window parent_window) {
+                         char *app_name, char *wm_role, Window parent_window) {
   void *p;
 
   if (parent_window) {
