@@ -151,35 +151,79 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
                                int double_draw_gap, int is_proportional, int is_var_col_width,
                                u_int32_t fg_pixel, u_int32_t bg_pixel,
                                int x, /* win->x and win->hmargin are added. */
-                               int y, int y_off, u_char **bitmaps, u_int len, u_char *picture,
+                               int y, /* win->y and win->vmargin are added. */
+                               u_char **bitmaps, u_int len, u_char *picture,
                                size_t picture_line_len, int src_bg_is_set, int bpp) {
+  int clip_y;
+  u_int clip_bottom;
   size_t size;
   u_char *src;
   u_char *p;
   u_int count;
+  int y_off;
   int orig_x;
-  int orig_y_off;
-  /*
-   * If font_width is larger than xfont->width_full, memory access must be limited
-   * to the latter one. (Don't excess xfont->glyph_width_bytes)
-   */
-  u_int max_glyph_width = xfont->width_full;
   u_char *bitmap_line;
   u_long pixel;
 
-  if (max_glyph_width + font_x_off > font_width) {
-    max_glyph_width = font_width - font_x_off;
+  /*
+   * Check if font->height or font->ascent excesses the display height,
+   * because font->height doesn't necessarily equals to the height of the
+   * US-ASCII font.
+   *
+   * XXX
+   * On the other hand, font->width is always the same (or exactly double) for now.
+   */
+
+  clip_y = win->y + win->vmargin + win->clip_y;
+  if (win->clip_height == 0) {
+    clip_bottom = win->y + ACTUAL_HEIGHT(win);
+  } else {
+    clip_bottom = clip_y + win->clip_height;
   }
 
-  /* (len + 1) is for decreasing the number of calling alloca() below. */
+  if (y >= clip_bottom) {
+    return;
+  } else if (y + font_height > clip_bottom) {
+    font_height = clip_bottom - y;
+  }
+
+  if (y + font_height < clip_y) {
+    return;
+  } else if (y < clip_y) {
+    y_off = clip_y - y;
+
+    if (picture) {
+      picture += (y_off * picture_line_len);
+    }
+  } else {
+    y_off = 0;
+  }
+
+  /*
+   * (len + 1) is for decreasing the number of calling alloca() below.
+   * (See if (retreat - filled > font_width) block.)
+   */
 #ifdef USE_FREETYPE
   if (xfont->is_aa && is_proportional) {
-    /* width_full == glyph_width_bytes / 3 */
-    u_int width = (len + 1) * (font_width > xfont->width_full ? font_width : xfont->width_full);
+    u_int width = font_width; /* means 'len + 1' */
 
+    for (count = 0; count < len; count++) {
+      if (bitmaps[count] && bitmaps[count][0] /* Glyph advance */ > font_width) {
+        width += bitmaps[count][0];
+      } else {
+        width += font_width;
+      }
+    }
+
+    /*
+     * The caller should check if the width of specified drawn strings <= the width
+     * of the window.
+     */
+#if 0
     if (x + width > win->width + win->hmargin + win->x) {
       width = win->width + win->hmargin + win->x - x;
     }
+#endif
 
     size = width * bpp;
 
@@ -217,18 +261,29 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
     return;
   }
 
-#if 1
-  /* Optimization for most cases */
-  if (src_bg_is_set && !double_draw_gap
 #ifdef USE_FREETYPE
-      && !xfont->face
+  if (!xfont->face || !xfont->is_aa)
 #endif
-      ) {
-    int x_off;
-    u_int end_gap = font_width - font_x_off - max_glyph_width;
-    u_int y_off_bytes = y_off * xfont->glyph_width_bytes;
+  {
+    u_int y_off_bytes;
+    /*
+     * If font_width is larger than xfont->width_full, memory access must be limited
+     * to the latter one. (Don't excess xfont->glyph_width_bytes)
+     */
+    u_int max_glyph_width = xfont->width_full;
 
-    switch (bpp) {
+    if (max_glyph_width + font_x_off > font_width) {
+      max_glyph_width = font_width - font_x_off;
+    }
+
+#if 1
+    /* Optimization for most cases */
+    if (src_bg_is_set && !double_draw_gap) {
+      int x_off;
+      u_int end_gap = font_width - font_x_off - max_glyph_width;
+      u_int y_off_bytes = y_off * xfont->glyph_width_bytes;
+
+      switch (bpp) {
       case 1:
         for (; y_off < font_height; y_off++, y_off_bytes += xfont->glyph_width_bytes) {
           p = (picture ? memcpy(src, (picture += picture_line_len), size)
@@ -307,23 +362,18 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
         }
 
         return;
+      }
     }
-  }
 #endif
 
-  orig_x = x;
-  orig_y_off = y_off;
+    orig_x = x;
 
-  for (; y_off < font_height; y_off++) {
-#if defined(USE_FREETYPE)
-    int prev_crowded_out = 0;
-#endif
-
-    if (src_bg_is_set) {
-      if (picture) {
-        memcpy(src, (picture += picture_line_len), size);
-      } else {
-        switch (bpp) {
+    for (; y_off < font_height; y_off++) {
+      if (src_bg_is_set) {
+        if (picture) {
+          memcpy(src, (picture += picture_line_len), size);
+        } else {
+          switch (bpp) {
           case 1:
             memset(src, bg_pixel, size);
             break;
@@ -336,14 +386,88 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
           default:
             memset32(src, bg_pixel, size / 4);
             break;
+          }
         }
       }
+
+      p = src;
+
+      y_off_bytes = y_off * xfont->glyph_width_bytes;
+
+      for (count = 0; count < len; count++, x += font_width) {
+        int x_off;
+
+        if (!ui_get_bitmap_line(xfont, bitmaps[count], y_off_bytes, bitmap_line)) {
+          if (src_bg_is_set) {
+            p += (font_width * bpp);
+          } else {
+            for (x_off = 0; x_off < font_width; x_off++, p += bpp) {
+              pixel = ui_display_get_pixel(win->disp, x + x_off, y + y_off);
+              copy_pixel(p, pixel, bpp);
+            }
+          }
+        } else {
+          int force_fg = 0;
+          u_int glyph_end = font_x_off + max_glyph_width;
+
+          for (x_off = 0; ; x_off++, p += bpp) {
+            if (font_x_off <= x_off && x_off < glyph_end &&
+                ui_get_bitmap_cell(bitmap_line, x_off - font_x_off)) {
+              pixel = fg_pixel;
+              force_fg = double_draw_gap;
+            } else if (font_width <= x_off) {
+              break;
+            } else {
+              if (force_fg) {
+                pixel = fg_pixel;
+                force_fg = 0;
+              } else if (src_bg_is_set) {
+                continue;
+              } else {
+                pixel = ui_display_get_pixel(win->disp, x + x_off, y + y_off);
+              }
+            }
+
+            copy_pixel(p, pixel, bpp);
+          }
+        }
+      }
+
+      ui_display_put_image(win->disp, (x = orig_x), y + y_off, src, p - src, !src_bg_is_set);
     }
-
-    p = src;
-
+  }
 #if defined(USE_FREETYPE)
-    if (xfont->is_aa) {
+  else {
+    int orig_y_off = y_off;
+
+    orig_x = x;
+
+    for (; y_off < font_height; y_off++) {
+      int prev_crowded_out = 0;
+
+      if (src_bg_is_set) {
+        if (picture) {
+          memcpy(src, (picture += picture_line_len), size);
+        } else {
+          switch (bpp) {
+          case 1:
+            memset(src, bg_pixel, size);
+            break;
+
+          case 2:
+            memset16(src, bg_pixel, size / 2);
+            break;
+
+          /* case  4: */
+          default:
+            memset32(src, bg_pixel, size / 4);
+            break;
+          }
+        }
+      }
+
+      p = src;
+
       if (is_proportional) {
         /*
          * Bitmap format
@@ -370,6 +494,12 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
               }
             }
 
+            if (prev_crowded_out > font_width) {
+              prev_crowded_out -= font_width;
+            } else {
+              prev_crowded_out = 0;
+            }
+
             x += font_width;
           } else {
             int retreat;
@@ -497,15 +627,46 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
               }
             } else {
               /* is_var_col_width is false */
-              int padding = font_x_off;
+              int padding;
 
               width += font_x_off;
-              if (width > (len - count) * font_width) {
-                width = (len - count) * font_width;
-              } else if (width + font_x_off < font_width) {
-                int add = (font_width - width - font_x_off) / 2;
-                width += add;
-                padding += add;
+
+              /*
+               * font_x_off is added to width above, so 'width + font_x_off'
+               * means that font_x_off * 2 is added to width.
+               */
+#ifdef CENTER_PROPORTIONAL_GLYPHS
+              if (width + font_x_off < font_width) {
+                int diff = (font_width - width - font_x_off) / 2;
+
+                width += diff;
+                padding = font_x_off + diff;
+              } else
+#endif
+              {
+                if (width + font_x_off > font_width) {
+                  int diff = (width + font_x_off - font_width) / 2;
+
+                  if (diff > font_x_off) {
+                    padding = 0;
+
+                    /*
+                     * If you use Inconsolata font with --otl option, === is shaped
+                     * with garbage by this.
+                     */
+#if 0
+                    bitmap_line += (diff - font_x_off) * 3;
+#endif
+                  } else {
+                    padding = font_x_off - diff;
+                  }
+                } else {
+                  padding = font_x_off;
+                }
+
+                if (width > (len - count) * font_width) {
+                  width = (len - count) * font_width;
+                }
               }
 
               if (padding > 0) {
@@ -535,7 +696,7 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
               if (src_bg_is_set /* && !picture */) {
                 for (; x_off < width; x_off++, p += bpp) {
                   if (copy_blended_pixel(win->disp->display, p, &bitmap_line, fg_pixel, bg_pixel,
-                                       bpp)) {
+                                         bpp)) {
                     /* do nothing */
                   } else if (x_off >= prev_crowded_out) {
                     copy_pixel(p, bg_pixel, bpp);
@@ -575,8 +736,6 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
           }
         }
       } else {
-        u_int y_off_bytes = y_off * xfont->glyph_width_bytes;
-
         /* is_proportional is false */
         for (count = 0; count < len; count++, x += font_width) {
           int x_off;
@@ -591,14 +750,35 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
               }
             }
           } else {
-            u_int glyph_end = font_x_off + max_glyph_width;
+            u_int glyph_end = font_x_off + ((u_int16_t*)bitmaps[count])[1];
+            int padding;
 
-            bitmap_line = bitmaps[count] + y_off_bytes;
+            /*
+             * font_x_off is added to width above, so 'width + font_x_off'
+             * means that font_x_off * 2 is added to width.
+             */
+#ifdef CENTER_PROPORTIONAL_GLYPHS
+            if (glyph_end + font_x_off < font_width) {
+              int diff = (font_width - glyph_end - font_x_off) / 2;
+
+              glyph_end += diff;
+              padding = font_x_off + diff;
+            } else
+#endif
+            {
+              padding = font_x_off;
+
+              if (glyph_end > font_width) {
+                glyph_end = font_width;
+              }
+            }
+
+            bitmap_line = bitmaps[count] + 4 + y_off * ((u_int16_t*)bitmaps[count])[0];
 
             if (src_bg_is_set) {
               if (picture) {
                 for (x_off = 0; ; x_off++, p += bpp) {
-                  if (font_x_off <= x_off && x_off < glyph_end) {
+                  if (padding <= x_off && x_off < glyph_end) {
                     u_long bg = (bpp == 2) ? *((u_int16_t *)p) : *((u_int32_t *)p);
                     copy_blended_pixel(win->disp->display, p, &bitmap_line, fg_pixel, bg, bpp);
                   } else if (font_width <= x_off) {
@@ -607,7 +787,7 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
                 }
               } else {
                 for (x_off = 0; ; x_off++, p += bpp) {
-                  if (font_x_off <= x_off && x_off < glyph_end) {
+                  if (padding <= x_off && x_off < glyph_end) {
                     copy_blended_pixel(win->disp->display, p, &bitmap_line, fg_pixel,
                                        bg_pixel, bpp);
                   } else if (font_width <= x_off) {
@@ -617,7 +797,7 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
               }
             } else {
               for (x_off = 0; ; x_off++, p += bpp) {
-                if (font_x_off <= x_off && x_off < glyph_end) {
+                if (padding <= x_off && x_off < glyph_end) {
                   u_long bg = ui_display_get_pixel(win->disp, x + x_off, y + y_off);
                   if (copy_blended_pixel(win->disp->display, p, &bitmap_line, fg_pixel, bg, bpp)) {
                     continue;
@@ -633,53 +813,11 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
           }
         }
       }
-    } else
-#endif
-    {
-      u_int y_off_bytes = y_off * xfont->glyph_width_bytes;
 
-      for (count = 0; count < len; count++, x += font_width) {
-        int x_off;
-
-        if (!ui_get_bitmap_line(xfont, bitmaps[count], y_off_bytes, bitmap_line)) {
-          if (src_bg_is_set) {
-            p += (font_width * bpp);
-          } else {
-            for (x_off = 0; x_off < font_width; x_off++, p += bpp) {
-              pixel = ui_display_get_pixel(win->disp, x + x_off, y + y_off);
-              copy_pixel(p, pixel, bpp);
-            }
-          }
-        } else {
-          int force_fg = 0;
-          u_int glyph_end = font_x_off + max_glyph_width;
-
-          for (x_off = 0; ; x_off++, p += bpp) {
-            if (font_x_off <= x_off && x_off < glyph_end &&
-                ui_get_bitmap_cell(bitmap_line, x_off - font_x_off)) {
-              pixel = fg_pixel;
-              force_fg = double_draw_gap;
-            } else if (font_width <= x_off) {
-              break;
-            } else {
-              if (force_fg) {
-                pixel = fg_pixel;
-                force_fg = 0;
-              } else if (src_bg_is_set) {
-                continue;
-              } else {
-                pixel = ui_display_get_pixel(win->disp, x + x_off, y + y_off);
-              }
-            }
-
-            copy_pixel(p, pixel, bpp);
-          }
-        }
-      }
+      ui_display_put_image(win->disp, (x = orig_x), y + y_off, src, p - src, !src_bg_is_set);
     }
-
-    ui_display_put_image(win->disp, (x = orig_x), y + y_off, src, p - src, !src_bg_is_set);
   }
+#endif
 }
 
 static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
@@ -691,13 +829,12 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
   u_char **bitmaps;
   u_int font_height;
   u_int font_ascent;
-  int y_off;
   u_char *picture;
   size_t picture_line_len;
   u_int count;
   int src_bg_is_set;
-  u_int clip_bottom;
   int use_ot_layout;
+  int get_space_glyph;
   int x_off;
 
   if (!win->is_mapped) {
@@ -717,46 +854,7 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
   use_ot_layout = 0;
 #endif
 
-  /*
-   * Check if font->height or font->ascent excesses the display height,
-   * because font->height doesn't necessarily equals to the height of the
-   *US-ASCII font.
-   *
-   * XXX
-   * On the other hand, font->width is always the same (or exactly double) for
-   *now.
-   */
-
-  font_ascent = font->ascent;
-  font_height = font->height;
-
-  if (win->clip_height == 0) {
-    clip_bottom = win->height;
-  } else {
-    clip_bottom = win->clip_y + win->clip_height;
-  }
-
-  if (y >= clip_bottom + font_ascent) {
-    return;
-  } else if (y + font_height - font_ascent > clip_bottom) {
-    font_height -= (y + font_height - font_ascent - clip_bottom + 1);
-
-    if (font_height < font_ascent) {
-      /* XXX I don't know why but without -1 causes line gap */
-      y -= (font_ascent - font_height - 1);
-      font_ascent = font_height;
-    }
-  }
-
-  if (y + font_height - font_ascent < win->clip_y) {
-    return;
-  } else if (y < win->clip_y + font_ascent) {
-    y_off = win->clip_y + font_ascent - y;
-  } else {
-    y_off = 0;
-  }
-
-/* Following check is done by the caller of this function. */
+  /* Following check is done by the caller of this function. */
 #if 0
   /*
    * Check if font->width * len excesses the display height
@@ -770,7 +868,7 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
 #endif
 
   x += (win->hmargin + win->x);
-  y = y + (win->vmargin + win->y) - font_ascent;
+  y = y + (win->vmargin + win->y) - font->ascent;
 
   if (wall_picture_bg) {
     /* bg_color is always NULL */
@@ -789,7 +887,7 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
     }
 
     picture_line_len = pic->width * bpp;
-    picture = pic->image + (pic_y + y_off) * picture_line_len +
+    picture = pic->image + pic_y * picture_line_len +
               /* - picture_line_len is for picture += picture_line_len below. */
               pic_x * bpp - picture_line_len;
 
@@ -805,10 +903,13 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
   }
 
   x_off = font->x_off;
+  /* See ui_calculate_char_width() in ui_font.c */
+  get_space_glyph = font->is_proportional && font->is_var_col_width;
 
   if (ch_len == 1) {
     for (count = 0; count < len; count++) {
-      bitmaps[count] = ui_get_bitmap(xfont, str + count, 1, use_ot_layout, NULL);
+      bitmaps[count] = ui_get_bitmap(xfont, str + count, 1, use_ot_layout,
+                                     get_space_glyph, NULL);
     }
   } else /* if(ch_len == 2) */ {
     XFontStruct *compl_xfont;
@@ -862,16 +963,22 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
       }
 
       compl_xfont = NULL;
-      bitmaps[count] = ui_get_bitmap(font->xfont, str, ch_len, use_ot_layout, &compl_xfont);
+      bitmaps[count] = ui_get_bitmap(font->xfont, str, ch_len, use_ot_layout,
+                                     get_space_glyph, &compl_xfont);
       if (compl_xfont && xfont != compl_xfont) {
         u_int w;
 
         if (count > 0) {
-          draw_string_intern(win, xfont, font_height, font_ascent, font->width, x_off,
-                             font->double_draw_gap, font->is_proportional, font->is_var_col_width,
-                             fg_color->pixel, bg_color ? bg_color->pixel : 0, x, y, y_off,
-                             bitmaps, count, picture, picture_line_len, src_bg_is_set, bpp);
+          font_ascent = font->ascent;
+          font_height = font->height;
+          if (ui_modify_bitmaps(xfont, bitmaps, count, &font_height, &font_ascent)) {
+            draw_string_intern(win, xfont, font_height, font_ascent, font->width, x_off,
+                               font->double_draw_gap, font->is_proportional, font->is_var_col_width,
+                               fg_color->pixel, bg_color ? bg_color->pixel : 0, x, y,
+                               bitmaps, count, picture, picture_line_len, src_bg_is_set, bpp);
+          }
           x += (font->width * count);
+
           bitmaps[0] = bitmaps[count];
           len -= count;
           count = 0;
@@ -901,10 +1008,14 @@ static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
     }
   }
 
-  draw_string_intern(win, xfont, font_height, font_ascent, font->width, x_off,
-                     font->double_draw_gap, font->is_proportional, font->is_var_col_width,
-                     fg_color->pixel, bg_color ? bg_color->pixel : 0, x, y, y_off,
-                     bitmaps, len, picture, picture_line_len, src_bg_is_set, bpp);
+  font_ascent = font->ascent;
+  font_height = font->height;
+  if (ui_modify_bitmaps(xfont, bitmaps, len, &font_height, &font_ascent)) {
+    draw_string_intern(win, xfont, font_height, font_ascent, font->width, x_off,
+                       font->double_draw_gap, font->is_proportional, font->is_var_col_width,
+                       fg_color->pixel, bg_color ? bg_color->pixel : 0, x, y,
+                       bitmaps, len, picture, picture_line_len, src_bg_is_set, bpp);
+  }
 }
 
 static int copy_area(ui_window_t *win, Pixmap src, PixmapMask mask, int src_x, /* can be minus */
@@ -1913,10 +2024,10 @@ size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parse
 #ifdef USE_WAYLAND
   *keysym = event->ksym;
   {
-    char buf[7]; /* UTF_MAX_SIZE + 1 */
+    u_char buf[7]; /* UTF_MAX_SIZE + 1 */
     size_t len = ui_display_get_utf8(buf, event->ksym);
 
-    if (len > 2) {
+    if (len >= 2) {
       if (seq_len >= len) {
         static ef_parser_t *utf8_parser;
 
@@ -2261,9 +2372,12 @@ void ui_window_draw_rect_frame(ui_window_t *win, int x1, int y1, int x2, int y2)
   ui_window_fill_with(win, &win->fg_color, x2, y1, 1, y2 - y1 + 1);
 }
 
+/* These functions are defined in ui_display.c on wayland. */
+#ifndef USE_WAYLAND
 void ui_set_use_clipboard_selection(int use_it) {}
 
 int ui_is_using_clipboard_selection(void) { return 0; }
+#endif
 
 int ui_window_set_selection_owner(ui_window_t *win, Time time) {
 #ifndef USE_SDL2
