@@ -373,7 +373,7 @@ typedef LPLOGFONT XFontSet;
 
 int XParseGeometry(char *str, int *x, int *y, unsigned int *width, unsigned int *height);
 
-KeySym XStringToKeysym(char *str);
+KeySym XStringToKeysym(const char *str);
 
 /* === Platform dependent options === */
 
