@@ -775,6 +775,18 @@ static void reset_input_focus(ui_window_t *win) {
   }
 }
 
+static ui_window_t *get_focused_child(ui_window_t *win) {
+  u_int count;
+
+  for (count = 0; count < win->num_children; count++) {
+    if (win->children[count]->is_focused) {
+      return win->children[count];
+    }
+  }
+
+  return NULL;
+}
+
 /* --- global functions --- */
 
 int ui_window_init(ui_window_t *win, u_int width, u_int height, u_int min_width, u_int min_height,
@@ -1663,6 +1675,8 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
                  */
                 kev.ch += 0x20;
               }
+            } else if (kev.state & ControlMask) {
+              /* WM_*_CHAR message will not be sent. */
             } else {
               /* wait for WM_*_CHAR message. */
               break;
@@ -1672,6 +1686,9 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
              * - See ui_xic_get_str() in win32/ui_xic.c.
              * - Control+0-9 doesn't cause WM_*_CHAR message.
              */
+          } else if ((kev.state & ShiftMask) && (kev.state & ControlMask) &&
+                     (kev.ch < 'A' || 'Z' < kev.ch)) {
+            /* WM_*_CHAR message will not be sent. */
           } else {
             /* wait for WM_*_CHAR message. */
             break;
@@ -1826,7 +1843,10 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
     case WM_MBUTTONDOWN:
     case WM_RBUTTONDOWN:
       if (!win->button_pressed) {
-        return 1;
+        /* Mouse wheel doesn't work on win8 without this. */
+        if ((win = get_focused_child(win)) == NULL) {
+          return 1;
+        }
       }
 
       goto BUTTON_MSG;
