@@ -36,7 +36,9 @@ extern "C" {
 #include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 
 #ifndef USE_WIN32API
+#ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE
+#endif
 #include <wchar.h> /* wcwidth */
 #endif
 }
@@ -45,6 +47,9 @@ extern "C" {
 #include <parser.h>
 #include <completeterminal.h>
 #include <networktransport.h>
+#ifdef MOSH_SIXEL
+#define USE_TCP_RECV_FROM_SERVER
+#endif
 #include <networktransport-impl.h>
 #include <terminaloverlay.h>
 #include <user.h>
@@ -170,7 +175,7 @@ static ssize_t lo_send_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
 }
 
 /* Same as vt_pty_ssh.cpp */
-static int _socketpair(int af, int type, int proto, SOCKET sock[2]) {
+static int _socketpair(int af, int type, int proto, int sock[2]) {
   SOCKET listen_sock;
   SOCKADDR_IN addr;
   int addr_len;
@@ -284,7 +289,7 @@ static int use_loopback(vt_pty_t *pty) {
   pty->stored->write = pty->write;
 
 #ifdef USE_WIN32API
-  if (_socketpair(AF_INET, SOCK_STREAM, 0, (SOCKET*)fds) == 0) {
+  if (_socketpair(AF_INET, SOCK_STREAM, 0, fds) == 0) {
     u_long val;
 
     val = 1;
@@ -688,7 +693,7 @@ static ssize_t write_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
         /* Quit sequence is Ctrl-^ . */
         if (pty_mosh->network->has_remote_addr()) {
           pty_mosh->overlay->get_notification_engine().set_notification_string(
-            wstring( L"Exiting on user request..." ), true);
+            std::wstring( L"Exiting on user request..." ), true);
           bl_trigger_sig_child(pty->child_pid);
         }
 
@@ -799,7 +804,7 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
                   pty_mosh->network->get_sent_state_acked_timestamp());
 #endif
 
-  size_t prev_len = min(pty_mosh->buf_len, len);
+  size_t prev_len = std::min(pty_mosh->buf_len, len);
 
   if (pty_mosh->buf_len > 0) {
     /*
@@ -860,10 +865,10 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
       if (!pass_seq_has_zmodem(&pty_mosh->network->ps) && memcmp(seq, "\x1bP", 2) == 0) {
         memcpy(buf, "\x1b[?8800h", 8); /* for DRCS-Sixel */
         len -= 8;
-        memcpy(buf + 8, seq, min(seq_len, len));
+        memcpy(buf + 8, seq, std::min(seq_len, len));
         prepend = 8;
       } else {
-        memcpy(buf, seq, min(seq_len, len));
+        memcpy(buf, seq, std::min(seq_len, len));
         prepend = 0;
       }
 
@@ -901,7 +906,7 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   pty_mosh->framebuffer = framebuffer;
   pty_mosh->initialized = true;
 
-  memcpy(buf, update.c_str(), min(update.length(), len));
+  memcpy(buf, update.c_str(), std::min(update.length(), len));
   if (update.length() > len) {
     if ((pty_mosh->buf = (char*)malloc(update.length() - len))) {
       pty_mosh->buf_len = update.length() - len;
