@@ -951,7 +951,7 @@ static ssize_t lo_send_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
 }
 
 /* Same as vt_pty_mosh.cpp */
-static int _socketpair(int af, int type, int proto, SOCKET sock[2]) {
+static int _socketpair(int af, int type, int proto, int sock[2]) {
   SOCKET listen_sock;
   SOCKADDR_IN addr;
   int addr_len;
@@ -1171,9 +1171,9 @@ static void *
   size_t rd_len;
   char buf[8192];
   int progress;
-  char msg1[] = "\x1b[?25l\r\nTransferring data.\r\n|";
+  const char msg1[] = "\x1b[?25l\r\nTransferring data.\r\n|";
   char *msg2;
-  char err_msg[] = "\r\nInterrupted.\x1b[?25h\r\n";
+  const char err_msg[] = "\r\nInterrupted.\x1b[?25h\r\n";
 
 #if !defined(USE_WIN32API) && defined(HAVE_PTHREAD)
   pthread_detach(pthread_self());
